//*****************************************************************************
//* A40 Driver PIC32MX575F512H
//* WRITTEN BY FRED EADY
//* NO USB SUPPORT IN THIS VERSION
//* COMPILED USING XC32 v1.40 - MPLABX v3.26
//* LAST UPDATE	04/25/2016
//* NOTES:
//*****************************************************************************
//*****************************************************************************
//* CONFIGURATION BITS
//*****************************************************************************
    #pragma config FNOSC    = PRIPLL        // Oscillator Selection
    #pragma config FPLLMUL  = MUL_20        // PLL Multiplier
    #pragma config FPLLIDIV = DIV_2         // PLL Input Divider
    #pragma config FPLLODIV = DIV_1         // PLL Output Divider
    #pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
    #pragma config FWDTEN   = OFF           // Watchdog Timer
    #pragma config WDTPS    = PS1           // Watchdog Timer Postscale
    #pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
    #pragma config OSCIOFNC = OFF           // CLKO Enable
    #pragma config POSCMOD  = HS            // Primary Oscillator
    #pragma config IESO     = OFF           // Internal/External Switch-over
    #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
    #pragma config CP       = OFF           // Code Protect
    #pragma config BWP      = OFF           // Boot Flash Write Protect
    #pragma config PWP      = OFF           // Program Flash Write Protect
    #pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
//*****************************************************************************
//* Internal Function Prototypes
//*****************************************************************************
#include <xc.h>
#include <sys/attribs.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <GenericTypeDefs.h>
#include "ccsMacros.h"
#include "portMacros.h"
#include "a40Driver.h"
#include "tdelay.h"

typedef struct
{
	BYTE	fint:1;
	BYTE	fbatl:1;
}FFLAGS;
FFLAGS flags;

#define grnON		0x0000
#define grnOFF		0x0001
#define redON		0x0002
#define redOFF		0x0003
#define OFF			0
#define ON			1
#define red			2
#define grn			3
#define alarm		4
#define test		5

//*****************************************************************************
//* USART RECEIVE BUFFERS SETUP
//* 2,4,8,16,32,64,128 or 256 bytes 
//*****************************************************************************
#define USART1_RX_BUFFER_SIZE  256      
#define USART1_RX_BUFFER_MASK ( USART1_RX_BUFFER_SIZE - 1 )
BYTE USART1_RxBuf[USART1_RX_BUFFER_SIZE];
BYTE USART1_RxTail;
BYTE USART1_RxHead;

#define USART2_RX_BUFFER_SIZE  256      
#define USART2_RX_BUFFER_MASK ( USART2_RX_BUFFER_SIZE - 1 )
BYTE USART2_RxBuf[USART2_RX_BUFFER_SIZE];
BYTE USART2_RxTail;
BYTE USART2_RxHead;

#define USART4_RX_BUFFER_SIZE  256      
#define USART4_RX_BUFFER_MASK ( USART4_RX_BUFFER_SIZE - 1 )
BYTE USART4_RxBuf[USART4_RX_BUFFER_SIZE];
BYTE USART4_RxTail;
BYTE USART4_RxHead;

#define USART5_RX_BUFFER_SIZE  256      
#define USART5_RX_BUFFER_MASK ( USART5_RX_BUFFER_SIZE - 1 )
BYTE USART5_RxBuf[USART5_RX_BUFFER_SIZE];
BYTE USART5_RxTail;
BYTE USART5_RxHead;

#define USART6_RX_BUFFER_SIZE  256      
#define USART6_RX_BUFFER_MASK ( USART6_RX_BUFFER_SIZE - 1 )
BYTE USART6_RxBuf[USART6_RX_BUFFER_SIZE];
BYTE USART6_RxTail;
BYTE USART6_RxHead;
//*****************************************************************************
//* SRAM ALLOCATIONS
//*****************************************************************************
BYTE scratch8;
BYTE bitein;
BYTE pktBuf[16];
BYTE okBuf[3];
BYTE rxBufLCD[33];

//char rxBuf5[256];
//char rxBuf2[256];
//char rxBuf4[256];
//char indx5,indx2,indx4;
//char fileName[16];

const BYTE lcdOK[] = ":>";
const BYTE atmelPrompt[] = "12>";
const BYTE atmelError[] = "!";
BYTE rxBufAtmel[1024];
BYTE rxBufProg[1024];
WORD idxBufAtmel;
//*****************************************************************************
// INT3 interrupt handler
// it is set at priority level 3 with software context saving
//*****************************************************************************
void __ISR(_EXTERNAL_3_VECTOR, IPL3SOFT) INT_Handler(void)
{
	grnLED_On;
	flags.fint = 1;
	tdelayms(10);
	IFS0CLR = 0x00008000; //0000 0000 0000 0000 1000 0000 0000 0000
}
//*****************************************************************************
// INT4 interrupt handler
// it is set at priority level 4 with software context saving
//*****************************************************************************
void __ISR(_EXTERNAL_4_VECTOR, IPL4SOFT) BATL_Handler(void)
{
	bluLED_Tog;
	flags.fbatl = 1;
	tdelayms(150);
	IFS0CLR = 0x00080000; //0000 0000 0000 1000 0000 0000 0000 0000
}
//*****************************************************************************
// UART 1 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_1_VECTOR, IPL2SOFT) IntUart1Handler(void)
{
		BYTE data,tmphead,tmptail;
		
		data = U1RXREG;                   // read the received data 
								// calculate buffer index 
		tmphead = ( USART1_RxHead + 1 ) & USART1_RX_BUFFER_MASK;
		USART1_RxHead = tmphead;        // store new index 
		
		if ( tmphead == USART1_RxTail )
		{
		// ERROR! Receive buffer overflow 
		}
								
		USART1_RxBuf[tmphead] = data;   // store received data in buffer 
		IFS0CLR = 0x08000000;//0000 1000 0000 0000 0000 0000 0000 0000
}
//*****************************************************************************
// UART 2 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_2_VECTOR, IPL2SOFT) IntUart2Handler(void)
{
		BYTE data,tmphead,tmptail;
		
		data = U2RXREG;                   // read the received data 
								// calculate buffer index 
		tmphead = ( USART2_RxHead + 1 ) & USART2_RX_BUFFER_MASK;
		USART2_RxHead = tmphead;        // store new index 
		
		if ( tmphead == USART2_RxTail )
		{
		// ERROR! Receive buffer overflow 
		}
                                
		USART2_RxBuf[tmphead] = data;   // store received data in buffer 
		IFS1CLR = 0x00000200;//0000 0000 0000 0000 0000 0010 0000 0000
}
//*****************************************************************************
// UART 4 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_4_VECTOR, IPL2SOFT) IntUart4Handler(void)
{
BYTE data,tmphead,tmptail;
		
		data = U4RXREG;                   // read the received data 
								// calculate buffer index 
		tmphead = ( USART4_RxHead + 1 ) & USART4_RX_BUFFER_MASK;
		USART4_RxHead = tmphead;        // store new index 
		
		if ( tmphead == USART4_RxTail )
		{
		// ERROR! Receive buffer overflow 
		}
								
		USART4_RxBuf[tmphead] = data;   // store received data in buffer 
		IFS2CLR = 0x00000010;
}
//*****************************************************************************
// UART 5 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_5_VECTOR, IPL2SOFT) IntUart5Handler(void)
{
BYTE data,tmphead,tmptail;
		
		data = U5RXREG;                   // read the received data 
								// calculate buffer index 
		tmphead = ( USART5_RxHead + 1 ) & USART5_RX_BUFFER_MASK;
		USART5_RxHead = tmphead;        // store new index 
		
		if ( tmphead == USART5_RxTail )
		{
		// ERROR! Receive buffer overflow 
		}
								
		USART5_RxBuf[tmphead] = data;   // store received data in buffer 
		IFS2CLR = 0x00000400; //0000 0000 0000 0000 0000 0100 0000 0000
}
//*****************************************************************************
// UART 6 interrupt handler
// it is set at priority level 2 with software context saving
//*****************************************************************************
void __ISR(_UART_6_VECTOR, IPL2SOFT) IntUart6Handler(void)
{
	BYTE data,tmphead,tmptail;
	//get the incoming byte
	data = U6RXREG;  
	//calculate buffer index 
	tmphead = ( USART6_RxHead + 1 ) & USART6_RX_BUFFER_MASK;
	//store new index
	USART6_RxHead = tmphead;         
	if ( tmphead == USART6_RxTail )
	{
	// ERROR! Receive buffer overflow 
	}
	//store received data in buffer 						
	USART6_RxBuf[tmphead] = data;
	//0000 0000 0000 0000 0000 0000 1000 0000
	IFS2CLR = 0x00000080; 
}
//******************************************************************
//*	USART1 Receive Character Function
//******************************************************************
BYTE recvchar1(void)
{
  BYTE tmptail1;
  // wait for incoming data 
  while ( USART1_RxHead == USART1_RxTail );
  // calculate buffer index 
  tmptail1 = ( USART1_RxTail + 1 ) & USART1_RX_BUFFER_MASK;
  // store new index 
  USART1_RxTail = tmptail1;
  return USART1_RxBuf[tmptail1];
}
//******************************************************************
//*	USART2 Receive Character Function
//******************************************************************
BYTE recvchar2(void)
{
  BYTE tmptail2;
  // wait for incoming data 
  while ( USART2_RxHead == USART2_RxTail );
  // calculate buffer index 
  tmptail2 = ( USART2_RxTail + 1 ) & USART2_RX_BUFFER_MASK;
  // store new index 
  USART2_RxTail = tmptail2;
  return USART2_RxBuf[tmptail2];
}
//******************************************************************
//*	USART4 Receive Character Function
//******************************************************************
BYTE recvchar4(void)
{
  BYTE tmptail4;
  // wait for incomming data
  while ( USART4_RxHead == USART4_RxTail );
  // calculate buffer index
  tmptail4 = ( USART4_RxTail + 1 ) & USART5_RX_BUFFER_MASK;
  // store new index
  USART4_RxTail = tmptail4;
  return USART4_RxBuf[tmptail4];
}
//******************************************************************
//*	USART5 Receive Character Function
//******************************************************************
BYTE recvchar5(void)
{
  BYTE tmptail5;
  // wait for incomming data
  while ( USART5_RxHead == USART5_RxTail );
  // calculate buffer index
  tmptail5 = ( USART5_RxTail + 1 ) & USART5_RX_BUFFER_MASK;
  // store new index
  USART5_RxTail = tmptail5;
  return USART5_RxBuf[tmptail5];
}
//******************************************************************
//*	USART6 Receive Character Function
//******************************************************************
BYTE recvchar6(void)
{
  BYTE tmptail6;
  //calculate buffer index
  tmptail6 = ( USART6_RxTail + 1 ) & USART6_RX_BUFFER_MASK;
  //store new index
  USART6_RxTail = tmptail6;
  //return the retrieved byte
  return USART6_RxBuf[tmptail6];
}
//******************************************************************
//*	USART1 Character Waiting Function
//******************************************************************
BYTE CharInQueue1(void)
{
  return(USART1_RxHead != USART1_RxTail);
}
//******************************************************************
//*	USART2 Character Waiting Function
//******************************************************************
BYTE CharInQueue2(void)
{
  return(USART2_RxHead != USART2_RxTail);
}
//******************************************************************
//*	USART4 Character Waiting Function
//******************************************************************
BYTE CharInQueue4(void)
{
  return(USART4_RxHead != USART4_RxTail);
}
//******************************************************************
//*	USART5 Character Waiting Function
//******************************************************************
BYTE CharInQueue5(void)
{
  return(USART5_RxHead != USART5_RxTail);
}
//******************************************************************
//*	USART6 Character Waiting Function
//******************************************************************
BYTE CharInQueue6(void)
{
  return(USART6_RxHead != USART6_RxTail);
}
//******************************************************************************
//* UART1 REDIRECTOR FOR USB PORTAL
//******************************************************************************
void _mon_putc(char c)
{
	U1TXREG = c;
	while(U1STAbits.TRMT == 0);
}
//******************************************************************************
//* CREATE NEW FILE
//******************************************************************************
void newFile(BYTE filenum)
{
    BYTE bitein;
    char cmdBuf[32];
    
    //send new file command
    sprintf(cmdBuf,"new microsd%1d.csv\r",filenum);
    sendStrUart2(cmdBuf);
    //wait for command mode prompt
    do{
    if(CharInQueue2())
    {
        bitein = recvchar2();
    }
    }while(bitein != '>');
}
//******************************************************************************
//* WRITE File
//******************************************************************************
void writeFile(BYTE filenum)
{
    char cmdBuf[32];
    BYTE bitein;
    
    //send write command
    sprintf(cmdBuf,"write microsd%1d.csv 0\r",filenum);
    sendStrUart2(cmdBuf);
    //wait for card to indicate file is open and ready for writing
    do{
        if(CharInQueue2())
        {
            bitein = recvchar2();
        }
      }while(bitein != '<');
      
    //put data to write here
    sendBiteUart2('A');
    sendBiteUart2(',');
    sendBiteUart2('B');
    sendBiteUart2(',');
    sendBiteUart2('C');
    sendBiteUart2(',');
    sendBiteUart2(0x01);
    //commit the data to the card
    sendBiteUart2(0x0D);
    //send an empty line to end the write
    sendBiteUart2(0x0D);
    //wait for command mode prompt
        do{
        if(CharInQueue2())
        {
            bitein = recvchar2();
        }
    }while(bitein != '>');
}
//******************************************************************************
//* GET FILE SIZE
//******************************************************************************
WORD getFileSize(BYTE fileNum)
{
    char cmdBuf[32];
    WORD timeoutms;
    WORD i,j;
    WORD fileSize;
    char sizeBuf[8];

    memset(sizeBuf,0x00,sizeof(sizeBuf));
   
    //send size command
    sprintf(cmdBuf,"size microsd%1d.csv\r",fileNum);
    sendStrUart2(cmdBuf);
    //wait for card to respond to read -  about 18mS
    i=0;
    timeoutms = 5000;
    do{
        if(CharInQueue2())
        {
            rxBufProg[i++] = recvchar2();
        }
        tdelayms(1);
      }while(timeoutms-- && (rxBufProg[i-1] != '>'));
      i = 0;
      j = 0;
      	do{
		if(isspace(rxBufProg[j]))
		{
			++j;
		}
	}while(isspace(rxBufProg[j]));

      do{
          sizeBuf[i++] = rxBufProg[j++];
      }while(rxBufProg[j-1] != 0x0D && rxBufProg[j-1] != 0x0A);
      sizeBuf[i-1] = 0x00;
      fileSize = atoi(sizeBuf);
      return(fileSize);
 }
//******************************************************************************
//* READ File
//******************************************************************************
void readFile(BYTE fileNum)
{
    char cmdBuf[32];
    WORD timeoutms;
    WORD i;
    WORD fileSize;
    BYTE bitein;
   
    fileSize = getFileSize(fileNum);
    
    memset(rxBufProg,0x00,sizeof(rxBufProg));
    //send read command
    sprintf(cmdBuf,"read microsd%1d.csv 0 %u 3\r",fileNum,fileSize);
    sendStrUart2(cmdBuf);
    //wait for card to respond to read -  about 18mS
    i=0;
    timeoutms = 5000;
    do{
        if(CharInQueue2())
        {
        	bitein = recvchar2();
            sendBiteUart1(bitein);
            rxBufProg[i++] = bitein;
        }
        tdelayms(1);
      }while(timeoutms-- && (rxBufProg[i-1] != '>'));
 }
//******************************************************************************
//* READ File - ASCII only
//******************************************************************************
void readFileAscii(BYTE filenum)
{
    char cmdBuf[32];
    WORD timeoutms;
    WORD i;
    BYTE bitein;
    
    memset(rxBufAtmel,0x00,sizeof(rxBufAtmel));
    //send read command
    sprintf(cmdBuf,"read microsd%1d.csv\r",filenum);
    sendStrUart2(cmdBuf);
    //wait for card to respond to read -  about 18mS
    i=0;
    timeoutms = 5000;
    do{
        if(CharInQueue2())
        {
        	bitein = recvchar2();
            sendBiteUart1(bitein);
            rxBufAtmel[i++] = bitein;

        }
        tdelayms(1);
      }while(timeoutms-- && (rxBufAtmel[i-1] != '>'));
 }
//******************************************************************************
//* LIST File
//******************************************************************************
void listFile(void)
{
    char cmdBuf[32];
    WORD timeoutms;
    WORD i;
    BYTE bitein;
    
    //send list file command
    sprintf(cmdBuf,"ls\r");
    sendStrUart2(cmdBuf);
    //wait for card to respond to ls 
    i=0;
    timeoutms = 5000;
    do{
        if(CharInQueue2())
        {
            do{
            bitein = recvchar2();
            sendBiteUart1(bitein);
            }while(CharInQueue2());
        }
        tdelayms(1);
      }while(timeoutms-- && (bitein != '>'));
 }
//*****************************************************************************
//* ALARM LED CONTROL
//*****************************************************************************
BYTE ledCntl(BYTE alarmtest, BYTE color, BYTE onoff)
{
	BYTE indx,rc;
	WORD timeoutms;
	
	sendBiteUart4(0xAA);
	sendBiteUart4(0x9E);
	switch(color)
	{
		case grn:
			switch(onoff)
			{
				case OFF:
					sendBiteUart4(make8(grnOFF,1)); 
					sendBiteUart4(make8(grnOFF,0));
				break;
				case ON:
					sendBiteUart4(make8(grnON,1)); 
					sendBiteUart4(make8(grnON,0));
				break;
			}
		break;
		case red:
			switch(onoff)
			{
				case OFF:
					sendBiteUart4(make8(redOFF,1)); 
					sendBiteUart4(make8(redOFF,0));
				break;
				case ON:
					sendBiteUart4(make8(redON,1)); 
					sendBiteUart4(make8(redON,0));
				break;
			}
		break;	
	}
	switch(alarmtest)
	{	
		case alarm:
			sendBiteUart4(0x01);
			sendBiteUart4(0xF5);
			sendBiteUart4(0x00);
			sendBiteUart4(0xA5);
		break;
		case test:
			sendBiteUart4(0x01);
			sendBiteUart4(0xF5);
			sendBiteUart4(0x00);
			sendBiteUart4(0xED);
		break;
	}
	
	sendBiteUart4(0xCC);
	sendBiteUart4(0x33);
	sendBiteUart4(0xC3);
	sendBiteUart4(0x3C);
	
	memset(okBuf,0x00,sizeof(okBuf));
	indx = 0;
	timeoutms = 5000;
	do{
		if(CharInQueue4())
		{
			do{
			okBuf[indx++] = recvchar4();
			//printf("%c",okBuf[indx-1]);
			}while(CharInQueue4());
		}
			tdelayms(1);
	}while(--timeoutms && strcmp(lcdOK,okBuf) !=0);
	
	if(timeoutms)
	{
		rc = 1;
	}
    return rc;
}
//*****************************************************************************
//* FIND LCD
//*****************************************************************************
BYTE findLCD(void)
{
	WORD timeoutms;
	BYTE rc,indx;
    BYTE bitein;
    
    rc = 0;
    indx = 0;
	
	sendBiteUart4(0xAA);
	sendBiteUart4(0x30);
	sendBiteUart4(0xCC);
	sendBiteUart4(0x33);
	sendBiteUart4(0xC3);
	sendBiteUart4(0x3C);
	
    timeoutms = 5000;
    do{
        if(CharInQueue4())
        {
        	bitein = recvchar4();
        }
            tdelayms(1);
    }while(--timeoutms && bitein != 0x30);
    
    if(timeoutms)
    {	
    	do{
    		if(CharInQueue4())
    		{
    			rxBufLCD[indx++] = recvchar4();
    		}
    	}while(CharInQueue4());
    	
    	if(rxBufLCD[0x00] == 'T' && \
    	   rxBufLCD[0x01] == 'o' && \
    	   rxBufLCD[0x0B] == 'R' && \
    	   rxBufLCD[0x11] == 0xCC && \
    	   rxBufLCD[0x14] == 0x3C && \
    	   rxBufLCD[0x16] == '>')
    	{
    		rc = 1;
            //printf("LCD found\r\n");
    	}
	}
    return rc;
}
//*****************************************************************************
//* IDLE FUNCTION
//*****************************************************************************
void idle(void)
{
    BYTE bitein;
    
	if(CharInQueue1())
	{
        bitein = recvchar1();
        printf("%c",bitein);
 	}
}
//******************************************************************************
//* INITIALIZE FUNCTION
//******************************************************************************
void init(void)
 {
    WORD  value;
    WORD i;
    WORD timeoutms;
    BYTE rc;
    
    // PIC32MX CPU Speed Optimizations (Cache/Wait States/Peripheral Bus Clks)
    // On reset, and after c-startup:
    // - Prefetch Buffer is disabled,
    // - I Cache is disabled,
    // - PFM wait States set to max setting (7 = too slow!!!)
    // - Data Memory SRAM wait states set to max setting (1 = too slow!!!)
    
    // PBCLK - already set to SYSCLK/1 via config settings
    // Data Memory SRAM wait states: Default Setting = 1; set it to 0
    BMXCONbits.BMXWSDRM = 0;
    // Flash PM Wait States: MX Flash runs at 2 wait states @ 80 MHz
    CHECONbits.PFMWS = 2;
    // Prefetch-cache: Enable prefetch for cacheable PFM instructions
    CHECONbits.PREFEN = 1;
    // JTAG: Disable on PORTA
    DDPCONbits.JTAGEN = 0;
    AD1PCFG = 0xFFFF;
    TRISBCLR = 0xC023; //1100 0000 0010 0011
    TRISDCLR = 0x000A; //0000 0000 0000 1010
    TRISECLR = 0x003C; //0000 0000 0011 1100
    TRISFCLR = 0x0020; //0000 0000 0010 0000
	
    gpio9LO;
    cancelHI;
    testHI;
    bluLED_Off;
    redLED_Off;
    grnLED_Off;
    bluLED_On;
    tdelayms(500);
    redLED_On;
    tdelayms(500);
    grnLED_On;
    tdelayms(500);
    grnLED_Off;
    tdelayms(500);
    redLED_Off;
    tdelayms(500);
    bluLED_Off;
    
    flags.fint = 0;
    flags.fbatl = 0;

    rstLO;
    tdelayms(100);
    rstHI;
    tdelayms(5000);
 
    //Initialize INT3
    INTCONSET = _INTCON_INT3EP_MASK; //trigger on rising edge
    //priority 3 sub priority 0 - 00011111 bits <28:26> sub bits <24:25>
    IPC3SET = 0x0C000000; //0000 1100 0000 0000 0000 0000 0000 0000
    IEC0SET = 0x00008000;
    IFS0CLR = 0x00008000; //0000 0000 0000 0000 1000 0000 0000 0000

    //Initialize INT4
    INTCONSET = _INTCON_INT4EP_MASK; //trigger on rising edge
    //priority 4 sub priority 0 - 00011111 bits <28:26> sub bits <24:25>
    IPC4SET = 0x10000000; //0001 0000 0000 0000 0000 0000 0000 0000
    IEC0SET = 0x00080000;
    IFS0CLR = 0x00080000; //0000 0000 0000 1000 0000 0000 0000 0000
    
    //Initialize UART1
    U1BRG = 520; // Set Baud rate 9600
    U1STA = 0;
    U1MODE = 0x00008000; // Enable UART for 8-none-1
    U1STASET = 0x00001400; // Enable Transmit and Receive
    //priority 2 sub priority 3 - 00011111 bits <4:2> sub bits <0:1>
    IPC6SET = 0x0000000B;
    IEC0SET = 0x08000000;
    IFS0CLR = 0x08000000;
    // flush receive buffer 
    USART1_RxTail = 0x00;
    USART1_RxHead = 0x00;

    //Initialize UART2
    U2BRG = 520; // Set Baud rate 9600
    U2STA = 0;
    U2MODE = 0x00008000; // Enable UART for 8-none-1    
    U2STASET = 0x00001400; // Enable Transmit and Receive
    //priority 2 sub priority 3 - 00011111 bits <4:2> sub bits <0:1>
    IPC8SET = 0x0000000B;
    IEC1SET = 0x00000200;
    IFS1CLR = 0x00000200;
    // flush receive buffer 
    USART2_RxTail = 0x00;
    USART2_RxHead = 0x00;
    
    //Initialize UART4
    U4BRG = 43; // Set Baud rate 115200
    U4STA = 0;
    U4MODE = 0x00008000; // Enable UART for 8-none-1
    U4STASET = 0x00001400; // Enable Transmit and Receive
    //priority 2 sub priority 3 - 00011111 bits <12:10> sub bits <9:8>
    IPC12SET = 0x00000B00;
    IEC2SET = 0x00000010;
    IFS2CLR = 0x00000010;
    //flush receive buffer 
    USART4_RxTail = 0x00;
    USART4_RxHead = 0x00;

    //Initialize UART5
    U5BRG = 43; // Set Baud rate 115200
    U5STA = 0;
    U5MODESET = 0x00008000; // disable UART for 8-none-1
    U5STASET = 0x00001400; // Enable Transmit and Receive
     //priority 2 sub priority 3 - 00011111 bits <26:28> sub bits <25:24>
    IPC12SET = 0x0B000000;//0000 1011 0000 0000 0000 0000 0000 0000
    IEC2SET = 0x00000400;
    IFS2CLR = 0x00000400;
    //flush receive buffer 
    USART5_RxTail = 0x00;
    USART5_RxHead = 0x00;
    
    //Initialize UART6
    U6BRG = 520; // Set Baud rate 9600
    U6STA = 0;
    U6MODESET = 0x00008000; // disable UART for 8-none-1
    U6STASET = 0x00001400; // Enable Transmit and Receive
     //priority 2 sub priority 3 - 00011111 bits <20:18> sub bits <17:16>
    IPC12SET = 0x000B0000;//0000 0000 0000 1011 0000 0000 0000 0000
    IEC2SET = 0x00000080;
    IFS2CLR = 0x00000080; //0000 0000 0000 0000 0000 0000 1000 0000
    //flush receive buffer 
    USART6_RxTail = 0x00;
    USART6_RxHead = 0x00;
    
    // Set Interrupt Controller for multi-vector machineMode 
    INTCONSET = _INTCON_MVEC_MASK;

    // Enable Interrupt Exceptions 
    // set the CP0 status IE bit high to turn on interrupts globally
    __builtin_enable_interrupts();
    
/*    
    do{
		rc = findLCD;
		tdelayms(1000);
	}while(rc == 0);

    ledCntl(alarm,grn,ON);
    ledCntl(test,red,ON);

    tdelayms(10);
    atmrstLO;
    bluLED_On;
    redLED_On;
    tdelayms(100);
    atmrstHI;
    i=0;
    timeoutms = 5000;
    do{
    if(CharInQueue2())
    {
         do{
            rxBufAtmel[i++] = recvchar2();
         }while(CharInQueue2());
    }
             tdelayms(1);

  }while(--timeoutms && strcmp(atmelPrompt,rxBufAtmel) != 0);
  
  if(timeoutms)
  {
	 for(i=0;i<15;++i)
	 {
		 bluLED_Tog;
		 tdelayms(25);
		 redLED_Tog;
		 tdelayms(25);
	 }
	 bluLED_Off;
	 redLED_Off;
 }
 else
 {  
	 di();
	 bluLED_Off;
	 redLED_Off;
	 do{
		 bluLED_Tog;
		 redLED_Tog;
		 tdelayms(100);
	 }while(1);
  }
*/
}
//******************************************************************************
// MAIN
//******************************************************************************
int main (void)
{
    BYTE bitein,i;
	init();
	do{
		if(flags.fint)
		{
			if(CharInQueue6())
			{
				for(i=0;i<16;i++)
				{
					if(CharInQueue6())
					{
						pktBuf[i] = recvchar6();
						printf("%c",pktBuf[i]);
					}
				}
			}
			flags.fint = 0;
		}
	
	//if(pktBuf[2] ==0x02)
	//{
	//	ledCntl(alarm,red,ON);
	//}
		
		if(flags.fbatl)
		{
			redLED_Tog;
			flags.fbatl = 0;
		}
				
	}while(1);
/*
//****************************
// TERMINAL INTERFACE
//****************************
    do{
	if(CharInQueue1())
	{
            do{
            bitein = recvchar1();
            sendBiteUart2(bitein);
            }while(CharInQueue1());
	}
	if(CharInQueue2())
	{
            do{
            bitein = recvchar2();
            sendBiteUart1(bitein);
            }while(CharInQueue2());
	}
    }while(1);
*/
}



